
from pathlib import Path
from gitignore_parser import parse_gitignore


class ProjectDir:

    def __init__(self, directory):
        self.directory = Path(directory)

    def list(self) -> list[str]:
        result = []
        gitignore = None
        gitignore_file = self.directory / '.gitignore'
        if gitignore_file.exists():
            gitignore = parse_gitignore(gitignore_file)
        for file_path in self.directory.rglob('*'):
            if '.git' in file_path.parts:
                continue
            if gitignore and gitignore(str(file_path)):
                continue
            result.append(str(file_path.relative_to(self.directory)))
        return result

    def read_file(self, file_path: str) -> str:
        path = self.directory / file_path
        return path.read_text()



import os
from pathlib import Path
from datetime import date
import sys
import yaml

import anthropic
from jinja2 import Template

from repozee import load_definition
from repozee.projectdir import ProjectDir


class AI:

    tools = yaml.safe_load(load_definition('tools.yml'))
    system_prompt = load_definition('system.md')
    user_prompt = Template(load_definition('user.md'))

    def __init__(self, directory):

        self.directory = directory
        self.client = anthropic.Anthropic()
        self.messages = []
        self.projectdir = ProjectDir(directory)

    def ask_ai(self):
        message = self.client.messages.create(
            model="claude-3-5-sonnet-20241022",
            max_tokens=4096,
            temperature=0,
            system=self.system_prompt,
            messages=self.messages,
            tools=self.tools)
        return message

    def loop(self):
        while True:
            print('-'*80)
            if self.messages:
                question = input("\nYou: ").strip()
                print()
            else:
                question = "List"
            self.messages.append(
                {
                    "role": "user",
                    "content": [
                        {
                            "type": "text",
                            "text": self.user_prompt.render(
                                question=question,
                                today=date.today())
                        }
                    ]
                }
            )
            message = self.ask_ai()
            while self.messages[-1]['role'] != 'assistant':
                for content in message.content:
                    if content.type == 'text':
                        print(content.text)
                        print()
                        self.messages.append({
                            "role": "assistant",
                            "content": content.text
                        })
                    elif content.type == 'tool_use':
                        if content.name == 'quit':
                            return
                        else:
                            tool = getattr(self.projectdir, content.name)
                            self.messages.append({
                                "role": "assistant",
                                "content": [dict(content)]
                            })
                            id = content.id
                            result = str(tool(**content.input))
                            self.messages.append({
                                "role": "user",
                                "content": [{
                                    "type": "tool_result",
                                    "tool_use_id": id,
                                    "content": result
                                }]
                            })
                            message = self.ask_ai()



from wizlib.command import WizCommand
from wizlib.parser import WizParser
from wizlib.ui import Choice, Chooser
from wizlib.command import CommandCancellation


class RepoZeeCommand(WizCommand):

    default = 'chat'



from pathlib import Path
from wizlib.parser import WizParser

from repozee.ai import AI
from repozee.command import RepoZeeCommand


class ChatCommand(RepoZeeCommand):

    name = 'chat'

    @classmethod
    def add_args(cls, parser: WizParser):
        super().add_args(parser)
        parser.add_argument('directory', default="", nargs='?')

    def handle_vals(self):
        super().handle_vals()
        if not self.provided('directory'):
            self.directory = str(Path.cwd())

    @RepoZeeCommand.wrap
    def execute(self):
        repozee = AI(self.directory)
        repozee.loop()
